/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "configuration.h"
#include "rng.h"
#include "swarm.h"
#include "problem.h"
#include "pso.h"
#include "localSearch.h"
#include "incrementalPSO.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

IncrementalPSO::IncrementalPSO(Configuration* config, Swarm* swarm, Problem* problem) : PSO(config,swarm,problem)
{	
	ls = new LocalSearch(problem, config);
}

IncrementalPSO::~IncrementalPSO(){
	delete ls;
}

bool IncrementalPSO::notDone(){
	//Check termination criterion
	if(p->getFunctionEvaluations() >= config->getMaxFES() || p->getBestSolutionValue() <= EPSILON)
		return false;
	return true;
}


void IncrementalPSO::run(){

	unsigned int iterations=0;
	Particle* particle;
	const double* velocityVector;
	const double* positionVector;
	const double* bestPositionVector;
	const double* neighborBestPositionVector;
	double newVelocity[config->getProblemDimension()];
	double newPosition[config->getProblemDimension()];
	std::vector<int> neighbors;

	double stepsize;  
	double before,after;
	long double bestSoFar = LDBL_MAX;
	unsigned int noImprovement=0;
	do{
		//Evaluation
		for(int i=0;i<s->getSize();i++)
			(s->getParticle(i))->evaluateCurrentPosition();					

		if(config->getLocalSearchID() != NO_LOCAL_SEARCH){
			//Set a random stepsize value
			stepsize = RNG::randVal(0,config->getMaxInitRange()-config->getMinInitRange()); 
			// get best particle
			particle = s->getBestParticle();
				if(s->getSize() == 1){ // if population size is equal to 1
					//apply local search to the best particle
					before = particle->getBestValue();
					ls->apply(particle,stepsize);				
					after = particle->getBestValue();
					//check the particle new position is better than the previous or not
					if(after < before)
						// if yes, reset failed attepmts
						particle->resetFailedAttempts();
					else 	// otherwise increment number of failed attempts
						particle->incrementFailedAttempts();
				}
				else{
					//Check the best particle is in local optima
					if(particle->getFailedAttempts()==config->getMaxRepeatedFailures()){	
						//if yes, apply local search to another (random) particle
						int pid = particle->getID();
						particle = s->getRandomParticle();
						while (particle->getID()==pid && particle->getFailedAttempts()>0)
							particle = s->getRandomParticle();	
					}
					// select a reference particle for determining step size adaptively
					Particle * reference = s->getRandomParticle();					
					while (reference->getID()==particle->getID())
						reference = s->getRandomParticle();
					
					// determine the max vector distance between the best(or random) and reference particle
					double max = 0.0;
					for(int j=0;j<config->getProblemDimension();j++){
						double temp = fabs(reference->getBestPosition()[j] - particle->getBestPosition()[j]);						
						if(temp > max)
							max = temp; 
					}
					// set step size as max vector distance 
					stepsize = max;							
					
					// if stepsize is smaller than EPSILON, set is as EPSILON
					if(stepsize <= EPSILON){
						stepsize = EPSILON;
					}

					before = particle->getBestValue();
					// apply local search
					ls->apply(particle,stepsize);				
					after = particle->getBestValue();
					
					//check the particle new position is better than the previous or not
					if(after < before)
						particle->resetFailedAttempts();				
					else
						particle->incrementFailedAttempts();
				}
		}
		
		// apply PSO rule to each particle
		for(int i=0;i<s->getSize();i++){
			particle = s->getParticle(i);
			
			neighborBestPositionVector = (s->getBestNeighborOf(particle))->getBestPosition();
			
			//Update velocity & positions
			velocityVector = particle->getCurrentVelocity();
			positionVector = particle->getCurrentPosition();
			bestPositionVector = particle->getBestPosition();
			

			double r1 = RNG::randVal(0.0,config->getAccCoef1());
			double r2 = RNG::randVal(0.0,config->getAccCoef2());

			// apply PSO rule to each dimension of particle position and velocity
			for(int j=0;j<config->getProblemDimension();j++){
				//Velocity update
				newVelocity[j] = config->getInertiaWeight()*velocityVector[j] + r1*(bestPositionVector[j]-positionVector[j]) + r2*(neighborBestPositionVector[j]-positionVector[j]);

				//Velocity restriction
				newVelocity[j] = limitVelocity(newVelocity[j]); //One of several posible velocity control mechanisms
				newPosition[j] = clampPositionVelocity(positionVector[j] + newVelocity[j],&newVelocity[j]);

			}
			// Update the new velocity and position of the particle
			particle->setCurrentVelocity(newVelocity);
			particle->setCurrentPosition(newPosition);

		}
		
		//Population growth and vertical social learning
		if(iterations>0 && iterations%config->getGrowthPeriod()==0 && s->getSize()<config->getMaximumPopulationSize()){
			s->addParticle(new Particle(config,p,s->getSize(),s->getBestParticle()));
		}

		//If improvement is lower than EPSILON (quite conservative)
	 	if(iterations > 0 && fabs(bestSoFar-p->getBestSolutionValue()) < EPSILON ){
                        noImprovement++;
                }
                else{
                        bestSoFar = p->getBestSolutionValue();
                        noImprovement=0;
                }

               if(noImprovement == config->getMaxStagnantIterations()){
			//destroy the current swarm 
			delete s;	
			//create a new swarm (Keeps best-so-far solution)
			s = new Swarm(config,p,p->getBestSoFarSolution(),p->getBestSolutionValue());
                       noImprovement=0;		
               }
		iterations++;
	}while(notDone());	
}
