/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "localSearch.h"
#include "localSearchProblems.h"
#include "particle.h"
#include "rng.h"
#include "powell.h"

#include <iostream>
#include <cfloat>
using namespace std;

LocalSearch::LocalSearch(Problem* problem, Configuration* config)
{
    this->config = config;
    this->problem = problem;
}

LocalSearch::~LocalSearch(){}


void LocalSearch::apply(Particle * part, double stepSize){

    const double* pos = part->getBestPosition();
    double x[config->getProblemDimension()];

    // create a temporary position vector for the particle position
    for(int i = 0; i < config->getProblemDimension(); i++)
            x[i] = pos[i];

    long double min_val=LDBL_MAX;  /* the minimum objective value, upon return */
    
    // required data for controling and printing progress inside of local search method  
    long double current[7] ={(long double)problem->getFunctionEvaluations(),problem->getBestSolutionValue(), EPSILON, config->getStartTime(),config->getCurrentPopulationSize(), config->getMinInitRange(),config->getMaxInitRange()};

    // get objective function
    obj_func func = getProblemFunc(config->getProblemID());

    // apply local search, returns the termination code, the result is stored in min_val and current[1]
    int ret = powell(config->getProblemDimension(), x, stepSize, config->getFTol(), config->getLsIterations(), func, current, &min_val);
    
    if(current[1] < EPSILON)
	current[1] = 0.0;

    // update current number of function evaluations
    problem->setFunctionEvaluations((unsigned int)current[0]);
    
    //update best solution value
    if(current[1] < problem->getBestSolutionValue()){
            problem->setBestSolutionValue(current[1]);
	    problem->setBestSoFarSolution(x);
    }

    //update best position and value of the particle
    if(part->getBestValue() > current[1]){
            part->setBestValue(current[1]);
            part->setBestPosition(x);
    }

    switch(ret){
	case FTOL_REACHED: { 
		part->setToImprove(false);
	}break;
       	case MAXEVAL_REACHED: {
		part->setToImprove(true);
	}break;
	default:{
		cout << "Error" << endl;
		exit(1);
	}
    }

}



obj_func LocalSearch::getProblemFunc(int funcID)
{

    obj_func func;

    switch (funcID){

            case SHIFTEDSPHERE:
                func = shiftedSphereFunction;
                break;
            case SCHWEFEL221:
                func = schwefel221Function;
                break;
            case SHIFTEDROSENBROCK:
                func = shiftedRosenbrockFunction;
                break;
            case SHIFTEDRASTRIGIN:
                func = shiftedRastriginFunction;
                break;
            case SHIFTEDGRIEWANK:
                func = shiftedGriewankFunction;
                break;
            case SHIFTEDACKLEY:
                func = shiftedAckleyFunction;
                break;
            case SCHWEFEL12:
                func = schwefel12Function;
                break;
            case SCHWEFEL222:
                func = schwefel222Function;
                break;
            case EXTENDEDF10:
                func = extendedF10Function;
                break;
            case BOHACHEVSKY:
                func = bohachevskyFunction;
                break;
            case SCHAFFER:
                func = schafferFunction;
                break;
            case H1:
                func = h1Function;
                break;
            case H2:
                func = h2Function;
                break;
            case H3:
                func = h3Function;
                break;
            case H4:
                func = h4Function;
                break;
            case H7:
                func = h7Function;
                break;
            case H8:
                func = h8Function;
                break;
            case H9:
                func = h9Function;
                break;
            case H10:
                func = h10Function;
                break;

            default:
                func = NULL;
                cout<<"Error: Unexpected function ID"<<endl;
        }

        return func;

}

