/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/


//Configuration
#include "configuration.h"

//Problems
#include "problem.h"

#include "shiftedsphere.h" 		//f1
#include "schwefel221.h" 		//f2
#include "shiftedrosenbrock.h" 		//f3
#include "shiftedrastrigin.h" 		//f4
#include "shiftedgriewank.h" 		//f5
#include "shiftedackley.h" 		//f6
#include "schwefel222.h" 		//f7
#include "schwefel12.h" 		//f8
#include "extendedF10.h" 		//f9
#include "bohachevsky.h" 		//f10
#include "schaffer.h" 			//f11
#include "h1.h"				//f12
#include "h2.h"				//f13
#include "h3.h"				//f14
#include "h4.h"				//f15
#include "h7.h"				//f16
#include "h8.h"				//f17
#include "h9.h"				//f18
#include "h10.h"			//f19

//Algorithms
#include "pso.h"
#include "incrementalPSO.h"

//Timers
#include <sys/time.h>
#include <sys/resource.h>

//C++ Libraries
#include <iostream>


using namespace std;


Problem* initializeProblem(Configuration * config){

    Problem* problem;

    //Problem initialization
	switch(config->getProblemID()){
		case SHIFTEDSPHERE:{
			problem = new ShiftedSphere(config);
		}break;
		case SCHWEFEL221:{
			problem = new Schwefel221(config);
		}break;
		case SHIFTEDROSENBROCK:{
			problem = new ShiftedRosenbrock(config);
		}break;
		case SHIFTEDRASTRIGIN:{
			problem = new ShiftedRastrigin(config);
		}break;
		case SHIFTEDGRIEWANK:{
			problem = new ShiftedGriewank(config);
		}break;
		case SHIFTEDACKLEY:{
			problem = new ShiftedAckley(config);
		}break;
		case SCHWEFEL222:{
			problem = new Schwefel222(config);
		}break;
		case SCHWEFEL12:{
			problem = new Schwefel12(config);
		}break;
		case EXTENDEDF10:{
			problem = new ExtendedF10(config);
		}break;
		case BOHACHEVSKY:{
			problem = new Bohachevsky(config);
		}break;
		case SCHAFFER:{
			problem = new Schaffer(config);
		}break;
		case H1:{
			problem = new Hybrid1(config);
		}break;
		case H2:{
			problem = new Hybrid2(config);
		}break;
		case H3:{
			problem = new Hybrid3(config);
		}break;
		case H4:{
			problem = new Hybrid4(config);
		}break;
		case H7:{
			problem = new Hybrid7(config);
		}break;
		case H8:{
			problem = new Hybrid8(config);
		}break;
		case H9:{
			problem = new Hybrid9(config);
		}break;
		case H10:{
			problem = new Hybrid10(config);
		}break;
	}

	return problem;

}

int main( int argc, char** argv) {	

	double stime;
	static struct timeval tp;	

	Problem* problem;

	//Command line parsing
	Configuration* config = new Configuration(argc,argv);

	//Random number generator
	RNG::initializeRNG(config->getRNGSeed());

    	//Initialize Problem
	problem = initializeProblem(config);

	Swarm* swarm = new Swarm(config,problem);

	//Start timer
	gettimeofday( &tp, NULL );
   	stime =(double) tp.tv_sec + (double) tp.tv_usec/1000000.0;
	config->setStartTime(stime);
	
	//PSO intialization	
	PSO* pso = new IncrementalPSO(config,swarm,problem);
	
	//PSO execution
	pso->run();

	//Memory release
	RNG::deallocateRNG();
	RNG::deallocatePermutation();
	delete swarm;
	delete problem;
	delete pso;
	delete config;

	return 0;
}
