/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "configuration.h"
#include "problem.h"
#include "particle.h"

#include <cstdlib>
#include <cfloat>
#include <iostream>
#include <vector>

using namespace std;

Particle::Particle(Configuration* config,Problem* problem, int identifier) {
	this->config = config;
	this->problem = problem;
	id=identifier;
		
	currentPosition = new double[config->getProblemDimension()];
	currentVelocity = new double[config->getProblemDimension()];
	bestPosition = new double[config->getProblemDimension()];

	//Initialize particle positions and velocity 
	for(unsigned int i=0;i<config->getProblemDimension();i++){
		currentPosition[i]=RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());
		currentVelocity[i] = 0.0;
		bestPosition[i]=currentPosition[i];

	}

	//Initialization
	currentValue=bestValue=LDBL_MAX;


	//Self-reference since the initial population size = 1
	neighbors.push_back(id);	
	
	toimprove = true;
	failedAttempts=0;

}

Particle::Particle(Configuration* config,Problem* problem, int identifier, Particle* bestParticle) {
	this->config = config;
	this->problem = problem;
	id=identifier;
		
	currentPosition = new double[config->getProblemDimension()];
	currentVelocity = new double[config->getProblemDimension()];
	bestPosition = new double[config->getProblemDimension()];

	double r = RNG::randVal(0.0,1.0); // the greater the lower limit, the closer to "copy" the strategy becomes
	
	//Initialize the velocity and position according to best particle position
	for(unsigned int i=0;i<config->getProblemDimension();i++){
		double randompos = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());
		const double best = bestParticle->getBestPosition()[i];
		currentPosition[i] =   randompos + r*(best - randompos);
		currentVelocity[i] = 0.0;
		bestPosition[i]=currentPosition[i];
	}

	//Initialization
	currentValue=bestValue=LDBL_MAX;

	//Self-reference 
	neighbors.push_back(id);	

	toimprove = true;
	failedAttempts=0;
}


Particle::Particle(Configuration* config,Problem* problem, int identifier, double* bestSoFarSolution,double bestSolutionValue) {
	this->config = config;
	this->problem = problem;
	id=identifier;
		
	currentPosition = new double[config->getProblemDimension()];
	currentVelocity = new double[config->getProblemDimension()];
	bestPosition = new double[config->getProblemDimension()];
	
	//initialize the position and velocity of the particle,
	//and set its best position with the best so far solution (after restarting)
	for(unsigned int i=0;i<config->getProblemDimension();i++){
		currentPosition[i] =   RNG::randVal(config->getMinInitRange(),config->getMaxInitRange()); 
		currentVelocity[i] = 0.0;
		bestPosition[i]=bestSoFarSolution[i];
	}

	//Initialization
	currentValue=LDBL_MAX;
	bestValue=bestSolutionValue;

	//Self-reference 
	neighbors.push_back(id);	

	toimprove = true;
	failedAttempts=0;
}


Particle::~Particle(){
	neighbors.clear();
	delete[] currentPosition;
	delete[] currentVelocity;
	delete[] bestPosition;
}

int Particle::getID(){
	return id;
}

void Particle::setID(int ID){
    id = ID;
}

int Particle::getDimension(){
	return config->getProblemDimension();
}

void Particle::resetTopology(){
	neighbors.clear();
	switch(config->getTopologyType()){
		case FC_TOPOLOGY:{
			for(int i=0;i<config->getCurrentPopulationSize();i++)
				neighbors.push_back(i);			
		}break;				
		case RING_TOPOLOGY:{
			if(id==0)
				neighbors.push_back(config->getCurrentPopulationSize()-1);
			else
				neighbors.push_back(id-1);
			neighbors.push_back(id);
			if(id==config->getCurrentPopulationSize()-1)	
				neighbors.push_back(0);
			else	
				neighbors.push_back(id+1);
		}break;	
	}	
}

std::vector<int> Particle::getNeighbors(){
	return neighbors;
}

unsigned int Particle::getNeighborhoodSize(){
	return neighbors.size();
}


void Particle::addNeighbor(int neighborID){

    bool isInsertedBefore = false;

    for(int i=0 ; i < neighbors.size(); i++){
        if(neighborID == neighbors[i])
            isInsertedBefore = true;
    }

    if(!isInsertedBefore)
        neighbors.push_back(neighborID);

}

void Particle::deleteNeighbors(){
    neighbors.clear();
}

void Particle::evaluateCurrentPosition(){
	// evaluate objective function
	currentValue = problem->getFunctionValue(currentPosition);

	//Update memory
	if(currentValue <= bestValue){
		updateBestPosition(currentValue);
       		toimprove = true;
		failedAttempts=0;
	}
}

const double* Particle::getCurrentPosition(){
	return currentPosition;
}


void Particle::setCurrentPosition(double* newPosition){
	for(unsigned int i=0;i<config->getProblemDimension();i++)
		currentPosition[i]=newPosition[i];
}


const double* Particle::getCurrentVelocity(){
	return currentVelocity;
}

void Particle::setCurrentVelocity(double* newVelocity){
	for(unsigned int i=0;i<config->getProblemDimension();i++)
		currentVelocity[i]=newVelocity[i];
}



const double* Particle::getBestPosition(){
	return bestPosition;
}

void Particle::setBestPosition(double* newBestPosition){
	for(unsigned int i=0;i<config->getProblemDimension();i++)
		bestPosition[i]=newBestPosition[i];
}


void Particle::updateBestPosition(long double newBest){
	
	setBestValue(newBest);//update best position value
	
	for(unsigned int i=0;i<config->getProblemDimension();i++)
		bestPosition[i]=currentPosition[i];

}


long double Particle::getCurrentValue(){
	return currentValue;
}

void Particle::setCurrentValue(long double newValue){
	currentValue=newValue;
}

long double Particle::getBestValue(){
	return bestValue;
}

void Particle::setBestValue(long double newBestValue){
	bestValue=newBestValue;
}

bool Particle::toImprove()
{
    return toimprove;
}

void Particle::setToImprove(bool val)
{
    toimprove = val;
}

unsigned int Particle::getFailedAttempts(){
	return failedAttempts;
}

void Particle::incrementFailedAttempts(){
	failedAttempts++;
}

void Particle::resetFailedAttempts(){
	failedAttempts=0;
}

void Particle::reset(){
	for(unsigned int i=0;i<config->getProblemDimension();i++){
		currentPosition[i]=RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());
		currentVelocity[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange()) - currentPosition[i];
		bestPosition[i]=currentPosition[i];
	}

	//Initialization
	bestValue=currentValue=LDBL_MAX;
	failedAttempts=0;
}

void Particle::print(){
	unsigned int i;
	cout << endl;
	cout << "Particle " << id << endl;
	cout << "Current value = " << currentValue << endl;
	cout << "Best value = " << bestValue << endl;
	cout << "Failed attempts " << failedAttempts<<endl;
}
