/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "configuration.h"
#include "problem.h"
#include "particle.h"
#include "swarm.h"

#include <iostream>
#include <vector>
#include <cstdlib>
#include <cmath>

using namespace std;

 Swarm::Swarm(Configuration* config,Problem* problem){

	this->config=config;
	this->problem = problem;
	
	//Initialize swarm
	for(unsigned int i= 0; i < config->getInitialPopulationSize(); i++){
		config->setCurrentPopulationSize(config->getCurrentPopulationSize()+1);
		particles.push_back(new Particle(config,problem,i));					
	}
}

Swarm::Swarm(Configuration* config,Problem* problem,double* bestSoFarSolution,double bestSolutionValue){
	this->config=config;
	this->problem = problem;
	
	//Initialize swarm (after restarting)
	for(unsigned int i= 0; i < config->getInitialPopulationSize(); i++){
		config->setCurrentPopulationSize(config->getCurrentPopulationSize()+1);
		if(i==0){
			particles.push_back(new Particle(config,problem,i,bestSoFarSolution,bestSolutionValue));	
		}
		else
			particles.push_back(new Particle(config,problem,i));					
	}
}


Swarm::~Swarm(){
	for(unsigned int i=0;i<particles.size();i++)
		delete particles.at(i);
	particles.clear();
	config->setCurrentPopulationSize(0);
}

int Swarm::getSize(){
	return particles.size();
}

Particle* Swarm::getParticle(int id){
	if(id < particles.size() && id >= 0)
		return particles.at(id);
	else{
		cerr << "Trying to access an inexistent particle" << endl;
		exit(-1);
	}
}

void Swarm::addParticle(Particle* newParticle){
	vector<Particle*>::iterator it;
	it = particles.begin();

	//Random insertion
	int pos = (int)floor(RNG::randVal(0,particles.size()));
	particles.insert (it+pos,newParticle);

	config->setCurrentPopulationSize(config->getCurrentPopulationSize()+1);

	//Reassigning ids and reinstantiating topologies
	for(unsigned int i=0;i<particles.size();i++){
		(particles.at(i))->setID(i);		
	}

	for(unsigned int i=0;i<particles.size();i++){
		(particles.at(i))->resetTopology();		
	}
}

vector<Particle*> Swarm::getParticles()
{
    return particles;
}


long double Swarm::getSolutionValue(){
	long double temp,solution;
	for(unsigned int i=0;i<particles.size();i++){
		temp = (particles.at(i))->getBestValue();
		if(i==0)
			solution=temp;
		else{
			if(temp < solution)
				solution=temp;			
		}
	}
	return solution;
}

const double* Swarm::getSolution(){
	long double temp,solution;
	int index;
	for(unsigned int i=0;i<particles.size();i++){
		temp = (particles.at(i))->getBestValue();
		if(i==0){
			solution=temp;
			index = i;
		}
		else {			
			if(temp < solution){
				solution=temp;
				index = i;
			}			
		}
	}
	return (particles.at(index))->getBestPosition();
}

Particle* Swarm::getBestParticle(){
	long double temp,solution;
	int index;
	for(unsigned int i=0;i<particles.size();i++){
		temp = (particles.at(i))->getBestValue();
		if(i==0){
			solution=temp;
			index = i;
		}
		else {			
			if(temp < solution){
				solution=temp;
				index = i;
			}			
		}
	}
	return particles.at(index);
}

Particle* Swarm::getRandomParticle(){
	int index = (int)floor(RNG::randVal(0,particles.size()));	
	return particles.at(index);
}


Particle* Swarm::getBestNeighborOf(Particle* p){
	int i;
	long double best,current;
	Particle* bestParticle;
	Particle* currentParticle;
	std::vector<int> neighbors = p->getNeighbors();

	for(i=0;i<neighbors.size();i++){
		if(i==0){
			bestParticle = particles.at(neighbors.at(i));
			best = bestParticle->getBestValue();
		}
		else{
			currentParticle = particles.at(neighbors.at(i));
			current = currentParticle->getBestValue();	

			if(current < best){
				bestParticle = currentParticle;
				best = current;
			}
		}
	}
	return bestParticle;
}

double Swarm::Edistance(double* a, const double* b){
	double d=0.0;
	for(int i=0;i<config->getProblemDimension();i++){
		d+=(a[i]-b[i])*(a[i]-b[i]);
	}
	return sqrt(d);
}

double Swarm::Dispersion(){
	double d=0.0;
	double* centroid = new double[config->getProblemDimension()];
	for(int i =0;i<config->getProblemDimension();i++){
		double sum=0.0;
		for(int j=0;j<particles.size();j++){		
			sum+=((particles.at(j))->getCurrentPosition())[i];
		}
		centroid[i]=sum/(double)particles.size();
	}
	
	for(int i=0;i<particles.size();i++){
		d+=Edistance(centroid,(particles.at(i))->getCurrentPosition());
	}
	delete [] centroid;

	return d/(double)particles.size();
}


void Swarm::print(){
	Particle* particle;
	cout << "Swarm" << endl;
	for(unsigned int i=0;i<particles.size();i++){
		particle=particles.at(i);
		particle->print();
	}
}
